--[[
file: main.lua -- Where all the magic really happens
by: Solsis00
for: ClickBoxHealer Unit Frames addition

This file contains all the core functions that do the real work, including all status changes post frame creation

**COMPLETE: Converted to local cbh table successfully.
]]--

local addon, cbh = ...



function cbh.PlayerFrameUpdate()
	cbh.PlayerName:SetText(cbh.playerinfo.name)
	cbh.PlayerHealth:SetText(tostring(cbh.playerinfo.health))
	cbh.PlayerRole:SetTexture("Rift", cbh.RoleImgs[cbh.playerinfo.role])

	cbh.nameCalc(cbh.playerinfo.name, nil, "player")

	if cbh.playerinfo.role then
		cbh.PlayerRole:SetVisible(true)
		cbh.PlayerRole:SetTexture("Rift", cbh.RoleImgs[cbh.playerinfo.role])
	else cbh.PlayerRole:SetVisible(false) end
	
	if cbh.playerinfo.mark then
		cbh.PlayerRaidMark:SetVisible(true)
		cbh.PlayerRaidMark:SetTexture("Rift", cbh.RaidMarkerImages[cbh.playerinfo.mark])
	else cbh.PlayerRaidMark:SetVisible(false) end
	
	for i = 1, 4 do
		if cbh.playerinfo.calling == cbh.Calling[i] then
			cbh.PlayerName:SetFontColor(cbhCallingColors[i].r, cbhCallingColors[i].g, cbhCallingColors[i].b, 1)
		end
	end

	local temptid = { [cbh.playerinfo.id] = cbh.playerinfo.health }
	-- cbh.UpdatePlayerHP(x, temptid)
end





--[[ XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX ]]--
function cbh.TargetUnitChange(units)
	cbh.TargetWindow:SetVisible(false)
	cbh.UnitStatus["target"] = nil
	if units and cbhTargetValues.enabled then --and not cbh.UnitStatus[units].istarget then
		cbh.TargetWindow:SetVisible(true)
		cbh.GetTargetInfo(units)
	end
end

function cbh.GetTargetInfo(uid)
	local tid = cbh.unitDetail(uid)
	if tid then
		cbh.UnitStatus["target"] = {
			id = tid.id,
			name = tid.name,
			istarget = true,
			isfriendly = tid.friendly,
			
			level = tid.level,
			tier = tid.tier,
			role = tid.role,
			calling = tid.calling,
			raidmark = tid.mark,
			
			aggro = tid.aggro,
			offline = tid.offline,
			dead = nil,
			los = nil,	-- tid.blocked
			outofrange = nil,
			warfront = nil,
			
			xcoord = tid.coordX,
			ycoord = tid.coordY,
			health = tid.health,
			healthmax = tid.healthMax,
			healthcap = tid.healthCap,
			mana = tid.mana,
			manamax = tid.manaMax,
			
			haswhitedebuff = nil,
			whitedebuffqty = 0,
			poisonqty = 0,
			curseqty = 0,
			diseaseqty = 0,
			hasdebuffs = false,
			hasburnout = false,
		}
		
		cbh.nameCalc(tid.name, nil, "target")

		if tid.role then
			cbh.TargetRole:SetVisible(true)
			cbh.TargetRole:SetTexture("Rift", cbh.RoleImgs[tid.role])
		else cbh.TargetRole:SetVisible(false) end
		
		if tid.mark then
			cbh.TargetRaidMark:SetVisible(true)
			cbh.TargetRaidMark:SetTexture("Rift", cbh.RaidMarkerImages[tid.mark])
		else cbh.TargetRaidMark:SetVisible(false) end
		
		if tid.relation == "friendly" then
			if tid.calling and tid.player then
				for i = 1, 4 do
					if tid.calling == cbh.Calling[i] then
						cbh.TargetName:SetFontColor(cbhCallingColors[i].r, cbhCallingColors[i].g, cbhCallingColors[i].b, 1)
					end
				end
			else
				cbh.TargetName:SetFontColor(.1, .5, 1)
			end
		elseif tid.relation == "hostile" then
			cbh.TargetName:SetFontColor(.8, 0, 0)
		else
			cbh.TargetName:SetFontColor(.8, .8, 0)
		end

		local temptid = { [uid] = cbh.UnitStatus["target"].health }
		cbh.UpdateTargetHP(x, temptid)
	end
end


function cbh.UpdateTargetHP(x, units)
	if cbh.UnitStatus["target"] and units[cbh.UnitStatus["target"].id] then
		local thealth = units[cbh.UnitStatus["target"].id]
		-- for k, v in pairs(units) do
		-- if k == cbh.UnitStatus["target"].id then
			local healthcalc = (thealth / cbh.UnitStatus["target"].healthmax)
			local healthtick = (-healthcalc * cbhTargetValues.fwidth)
			if healthtick < cbhTargetValues.fwidth then healthtick = healthtick + cbhTargetValues.fwidth end
			cbh.TargetHP:SetWidth(healthtick)
			-- if cbhValues.healthgradient then
				if cbhValues.debuffcolorwhole == false or cbh.UnitStatus["target"].hasdebuffs == false then
					if healthcalc < .3 then
						cbh.TargetFrame:SetBackgroundColor(1, 0, 0)
					elseif healthcalc < .65 then
						cbh.TargetFrame:SetBackgroundColor(1, 1, 0)
					else cbh.TargetFrame:SetBackgroundColor(cbhCallingColors[7].r, cbhCallingColors[7].g, cbhCallingColors[7].b, 1) end
				end
			-- end
			if cbhValues.hidehealth == false then
				local health = 0
				local healthpercent = math.ceil(healthcalc * 100)
				if thealth > 1000000 then
					health = cbh.round((thealth/1000000), 1)
					health = tostring(health).."m"
					cbh.TargetHealth:SetText(health.."    "..healthpercent.."%")
				elseif thealth > 1000 then
					health = cbh.round((thealth/1000), 1)
					health = tostring(health).."k"
					cbh.TargetHealth:SetText(health.."    "..healthpercent.."%")
				elseif thealth <= 0 or math.ceil(healthcalc) == 0 then
					cbh.TargetHealth:SetText("DEAD")
					cbh.UnitStatus["target"].dead = true
				end
			end
		-- end
	-- end
	end
end


function cbh.UpdateTargetHPMax(hEvent, units)
	if cbh.UnitStatus["target"] and units[cbh.UnitStatus["target"].id] then
		local tmaxhp = units[cbh.UnitStatus["target"].id]
		cbh.UnitStatus["target"].healthmax = tmaxhp
		local tempid = {[cbh.UnitStatus["target"].id] = cbh.UnitStatus["target"].health}
		cbh.UpdateTargetHP(x, tempid)
	end
end


function cbh.UpdateTargetHPCap(bEvent, units)

end



function cbh.UpdateTargetMB(hEvent, units)

end


function cbh.UpdateTargetMBMax(hEvent, units)

end



function cbh.TargetMentoredUpdate(hEvent, units)
-- just send this back through the change target function
end



function cbh.TargetaggroUpdate(hEvent, units)

end



function cbh.TargetOnlineStatusUpdate(hEvent, units)

end



-- Triggered when absorption value on a unit changes
function cbh.TargetAbsorbUpdate(hEvents, units)

end



function cbh.TargetRaidMarkUpdate(hEvent, units)
	if cbh.UnitStatus["target"] and units[cbh.UnitStatus["target"].id] then
		local trmark = units[cbh.UnitStatus["target"].id]
		cbh.TargetRaidMark:SetVisible(true)
		cbh.TargetRaidMark:SetTexture("Rift", cbh.RaidMarkerImages[trmark])
	end
end



-- Updates group role changes based on current spec unless in a dungeon
function cbh.TargetRoleCheck(hEvent, units)

end



function cbh.TargetBuffAdding(hEvent, unit, buffs)

end


function cbh.TargetBuffRemoving(hEvent, unit, buffs)

end


function cbh.TargetHideOutofCombat(groupnum, oor)

end


function cbh.TargetUnitMouseOver(units)
	-- dump(units)
end

function cbh.UnitSlashCommands(hEvent, com)
	-- dump(hEvent,com)
end





--[[
find:			(UnitStatus.*\])\[1\]
replace:	\1\.aggro
]]--





-- table.insert(Library.LibUnitChange.Register("mouseover"), {cbh.PTUnitMouseOver, "ClickBoxHealer", "cbhOnUnitMouseOver"}) -- create a mouseover event
table.insert(Library.LibUnitChange.Register("player.target"), {cbh.TargetUnitChange, "ClickBoxHealer", "TargetUnitChange"})
Command.Event.Attach(Event.Unit.Detail.Health, cbh.UpdateTargetHP, "UpdateTargetHP")
Command.Event.Attach(Event.Unit.Detail.HealthMax, cbh.UpdateTargetHPMax, "UpdateHP")
Command.Event.Attach(Event.Unit.Detail.Mark, cbh.TargetRaidMarkUpdate, "UpdateRaidMark")


Command.Event.Attach(Command.Slash.Register("cbhu"), cbh.UnitSlashCommands, "SlashCommands")



-- Command.Event.Attach(Event.Unit.Detail.HealthMax, cbh.UpdateHPMax, "UpdateHP")
-- Command.Event.Attach(Event.Unit.Detail.HealthCap, cbh.UpdateHPCap, "UpdateHP")
-- Command.Event.Attach(Event.Unit.Detail.Mana, cbh.UpdateMB, "UpdateMB")
-- Command.Event.Attach(Event.Unit.Detail.ManaMax, cbh.UpdateMBMax, "UpdateMaxMana")
-- Command.Event.Attach(Event.Unit.Detail.Mentoring, cbh.Mentored, "UpdateMaxMana")


-- Command.Event.Attach(Event.Unit.Detail.Aggro, cbh.aggroUpdate, "UpdateArgo")
-- Command.Event.Attach(Event.Unit.Detail.Offline, cbh.OnlineStatusUpdate, "UpdateOffline")
-- Command.Event.Attach(Event.Unit.Detail.Absorb, cbh.Absorb, "UpdateAbsorbDetails")
-- Command.Event.Attach(Event.Unit.Detail.Mark, cbh.RaidMark, "UpdateRaidMark")
-- Command.Event.Attach(Event.Unit.Detail.Role, cbh.UnitRoleCheck, "UpdateRole")

-- Command.Event.Attach(Event.Buff.Add, cbh.BuffAdding, "TESTUpdateBuffs")
-- Command.Event.Attach(Event.Buff.Remove, cbh.BuffRemoving, "TESTRemoveBuffs")


-- Command.Event.Attach(Command.Slash.Register("cbhu"), cbh.SlashCommand, "SlashCommands")


