--[[
file: main.lua -- Where all the magic really happens
by: Solsis00
for: ClickBoxHealer Unit Frames addition

This file contains all the core functions that do the real work, including all status changes post frame creation

**COMPLETE: Converted to local cbh table successfully.
]]--

local addon, cbh = ...


function cbh.PlayerLoadSettings(x, addonid)
	cbh.defaults.PlayerValues = {
		enabled = true,
		fx = 800,
		fy = 900,
		fheight = 55,
		fwidth = 200,
		mfheight = 8,
		mfwidth = 200,
		cfheight = 8,
		cfwidth = 200,

		texture = 13,
		fontsize = 12,
		rolesize = 24,
		marksize = 38,

		namefontsize = 16,
		macros = true,
	}
	if not cbhPlayerValues then
		cbhPlayerValues = {}
		for k, v in pairs(cbh.defaults.PlayerValues) do
			cbhPlayerValues[k] = v
		end
	else
		for k, v in pairs(cbh.defaults.PlayerValues) do
			if cbhPlayerValues[k] == nil then
				cbhPlayerValues[k] = v
			end
		end
	end
end


--[[<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>]]
--				PLAYER FRAME SETUP
--[[<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>]]
cbh.NameGlowTable = { blurX=3, blurY=3, colorA=1, colorB=0, colorG=0, colorR=0, knockout=false, offsetX=0, offsetY=0, replace=false, strength=5 }
cbh.PlayerTable = {}
-- cbh.TargetClick = UI.CreateFrame("Frame", "TargetFrame", cbh.Context)
cbh.PlayerWindow = UI.CreateFrame("Frame", "PlayerWindow", cbh.Context)
cbh.PlayerWindowDrag = UI.CreateFrame("Texture", "PlayerDragFrame", cbh.PlayerWindow)
cbh.PlayerFrame = UI.CreateFrame("Texture", "PlayerFrame", cbh.PlayerWindow)
cbh.PlayerHP = UI.CreateFrame("Frame", "PlayerHP", cbh.PlayerWindow)
cbh.PlayerChargeFrame = UI.CreateFrame("Texture", "PlayerChargeFrame", cbh.PlayerWindow)
cbh.PlayerCharge = UI.CreateFrame("Frame", "PlayerCharge", cbh.PlayerWindow)
cbh.PlayerManaFrame = UI.CreateFrame("Texture", "PlayerManaFrame", cbh.PlayerWindow)
cbh.PlayerMana = UI.CreateFrame("Frame", "PlayerMana", cbh.PlayerWindow)

cbh.PlayerName = UI.CreateFrame("Text", "PlayerFrame", cbh.PlayerWindow)
cbh.PlayerHealth = UI.CreateFrame("Text", "PlayerHealth", cbh.PlayerWindow)
cbh.PlayerRole = UI.CreateFrame("Texture", "PlayerRole", cbh.PlayerWindow)
cbh.PlayerRaidMark = UI.CreateFrame("Texture", "PlayerRaidMark", cbh.PlayerWindow)
cbh.PlayerCharges = UI.CreateFrame("Text", "PlayerCharges", cbh.PlayerWindow)
cbh.PlayerCombat = UI.CreateFrame("Texture", "PlayerCombatIcon", cbh.PlayerWindow)



--[[<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>]]
--				PLAYER FRAME CREATION
--[[<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>]]
function cbh.CreatePlayerFrame()
	cbh.PlayerWindow:SetPoint("TOPLEFT", cbh.Context, "TOPLEFT", cbhPlayerValues.fx, cbhPlayerValues.fy)

	-- CREATE "DRAG ORB" FOR MOVING FRAME
	cbh.PlayerWindowDrag:SetPoint("TOPRIGHT", cbh.PlayerWindow, "TOPLEFT", 0, 0)
	-- cbh.WindowDrag:SetTexture("ClickBoxHealer", "Textures/lockorb.png")
	cbh.PlayerWindowDrag:SetTexture("Rift", "StarFlare_Orange.png.dds")
	cbh.PlayerWindowDrag:SetLayer(9)
	cbh.PlayerWindowDrag:SetWidth(48)
	cbh.PlayerWindowDrag:SetHeight(48)
	cbh.PlayerWindowDrag:SetVisible(false)

	-- ALLOWS DRAGGING OF FRAME AND REPOSITIONING WHEN FRAME ISN'T LOCKED
	cbh.PlayerWindowDrag:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		self.windowdragActive = true
		local mouseStatus = Inspect.Mouse()
		self.oldx = mouseStatus.x - cbhPlayerValues.fx
		self.oldy = mouseStatus.y - cbhPlayerValues.fy
	end, "Event.UI.Input.Mouse.LeftDown")

	--CHANGE SAVED FRAMENUMBERIABLES WHEN MOUSE IS MOVED WHILE DRAGGING ACTIVE
	cbh.PlayerWindowDrag:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.windowdragActive == true then
			self.moved = true
			local newx, newy
			local mouseStatus = Inspect.Mouse()
			cbhPlayerValues.fx = mouseStatus.x - self.oldx
			cbhPlayerValues.fy = mouseStatus.y - self.oldy
			cbh.PlayerWindow:SetPoint("TOPLEFT", cbh.Context, "TOPLEFT", cbhPlayerValues.fx, cbhPlayerValues.fy)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	-- ENDS DRAGGING WHEN LEFT CLICK RELEASED
	cbh.PlayerWindowDrag:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		self.windowdragActive = false
	end, "Event.UI.Input.Mouse.LeftUp")

	cbh.PlayerWindowDrag:EventAttach(Event.UI.Input.Mouse.Left.Upoutside, function(self, h)
		self.windowdragActive = false
	end, "Event.UI.Input.Mouse.Left.Upoutside")

	-- LOCKS FRAME IF RIGHT CLICKED
	cbh.PlayerWindowDrag:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		cbh.FrameLocked()
	end, "Event.UI.Input.Mouse.Right.Click")

	cbh.PlayerFrame:SetPoint("TOPLEFT", cbh.PlayerWindow, "TOPLEFT", 0, 0)
	cbh.PlayerFrame:SetTexture("ClickBoxHealer", "Textures/"..cbhPlayerValues.texture)
	-- cbh.PlayerFrame:SetTexture("ClickBoxHealer", "Textures/health_"..cbhPlayerValues.texture..".png")
	cbh.PlayerFrame:SetLayer(0)
	cbh.PlayerFrame:SetWidth(cbhPlayerValues.fwidth)
	cbh.PlayerFrame:SetHeight(cbhPlayerValues.fheight)
	-- cbh.PlayerFrame:SetBackgroundColor(0,0,0,1)
	-- cbh.PlayerFrame:SetAlpha(1)

	cbh.PlayerHP:SetPoint("TOPRIGHT", cbh.PlayerFrame, "TOPRIGHT", 0, 0)
	cbh.PlayerHP:SetLayer(1)
	cbh.PlayerHP:SetWidth(0)
	cbh.PlayerHP:SetHeight(cbhPlayerValues.fheight)
	cbh.PlayerHP:SetBackgroundColor(0,0,0,1)


	cbh.PlayerManaFrame:SetPoint("TOPLEFT", cbh.PlayerFrame, "BOTTOMLEFT", 0, 0)
	cbh.PlayerManaFrame:SetTexture("ClickBoxHealer", "Textures/health_1.png")
	cbh.PlayerManaFrame:SetLayer(0)
	cbh.PlayerManaFrame:SetWidth(cbhPlayerValues.fwidth)
	cbh.PlayerManaFrame:SetHeight(cbhPlayerValues.mfheight)
	cbh.PlayerManaFrame:SetBackgroundColor(0,0,.75,1)
	cbh.PlayerManaFrame:SetVisible(false)

	cbh.PlayerMana:SetPoint("TOPRIGHT", cbh.PlayerManaFrame, "TOPRIGHT", 0, 0)
	cbh.PlayerMana:SetLayer(1)
	cbh.PlayerMana:SetWidth(0)
	cbh.PlayerMana:SetHeight(cbhPlayerValues.mfheight)
	cbh.PlayerMana:SetBackgroundColor(0,0,0,1)


	cbh.PlayerChargeFrame:SetPoint("TOPLEFT", cbh.PlayerManaFrame, "BOTTOMLEFT", 0, 0)
	cbh.PlayerChargeFrame:SetTexture("ClickBoxHealer", "Textures/health_1.png")
	cbh.PlayerChargeFrame:SetLayer(0)
	cbh.PlayerChargeFrame:SetWidth(cbhPlayerValues.fwidth)
	cbh.PlayerChargeFrame:SetHeight(cbhPlayerValues.cfheight)
	cbh.PlayerChargeFrame:SetBackgroundColor(0.25,.5,.5,1)
	cbh.PlayerChargeFrame:SetVisible(false)

	cbh.PlayerCharge:SetPoint("TOPRIGHT", cbh.PlayerChargeFrame, "TOPRIGHT", 0, 0)
	cbh.PlayerCharge:SetLayer(1)
	cbh.PlayerCharge:SetWidth(0)
	cbh.PlayerCharge:SetHeight(cbhPlayerValues.cfheight)
	cbh.PlayerCharge:SetBackgroundColor(0,0,0,1)


	cbh.PlayerName:SetPoint("CENTER", cbh.PlayerFrame, "CENTER", 0, 0)
	cbh.PlayerName:SetLayer(2)
	cbh.PlayerName:SetFontSize(cbhPlayerValues.namefontsize)
	cbh.PlayerName:SetEffectGlow(cbh.NameGlowTable)

	cbh.PlayerHealth:SetPoint("BOTTOMCENTER", cbh.PlayerFrame, "BOTTOMCENTER", 0, 0)
	cbh.PlayerHealth:SetLayer(2)
	cbh.PlayerHealth:SetFontSize(cbhPlayerValues.fontsize)
	cbh.PlayerHealth:SetEffectGlow(cbh.NameGlowTable)

	-- cbh.PlayerCharges:SetPoint("BOTTOMLEFT", cbh.PlayerFrame, "BOTTOMLEFT", 0, 0)
	cbh.PlayerCharges:SetPoint("TOPRIGHT", cbh.PlayerFrame, "TOPRIGHT", 0, 0)
	cbh.PlayerCharges:SetLayer(2)
	cbh.PlayerCharges:SetFontSize(cbhPlayerValues.fontsize)

	cbh.PlayerRole:SetPoint("TOPLEFT", cbh.PlayerFrame, "TOPLEFT", 0, 0)
	cbh.PlayerRole:SetLayer(2)
	cbh.PlayerRole:SetWidth(cbhPlayerValues.rolesize)
	cbh.PlayerRole:SetHeight(cbhPlayerValues.rolesize)

	cbh.PlayerCombat:SetPoint("BOTTOMRIGHT", cbh.PlayerFrame, "TOPLEFT", 50, 10)
	cbh.PlayerCombat:SetTexture("ClickBoxHealer", "textures/combat.png")
	cbh.PlayerCombat:SetLayer(2)
	cbh.PlayerCombat:SetWidth(cbhPlayerValues.rolesize)
	cbh.PlayerCombat:SetHeight(cbhPlayerValues.rolesize)
	cbh.PlayerCombat:SetVisible(false)

	cbh.PlayerRaidMark:SetPoint("CENTER", cbh.PlayerFrame, "TOPCENTER", 0, 0)
	cbh.PlayerRaidMark:SetLayer(2)
	cbh.PlayerRaidMark:SetVisible(false)
	cbh.PlayerRaidMark:SetWidth(cbhPlayerValues.marksize)
	cbh.PlayerRaidMark:SetHeight(cbhPlayerValues.marksize)

	-- cbh.PlayerClick:SetPoint("TOPLEFT", cbh.Context, "TOPLEFT", cbhPlayerValues.fx, cbhPlayerValues.fy)
	-- cbh.PlayerClick:SetWidth(cbhPlayerValues.fwidth)
	-- cbh.PlayerClick:SetHeight(cbhPlayerValues.fheight)
	-- cbh.PlayerClick:SetAlpha(0)

	-- cbh.PlayerFrame:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		-- local tuid = cbh.unitDetail("player") end, "Event.UI.Input.Mouse.Left.Down")
	cbh.PlayerFrame:EventAttach(Event.UI.Input.Mouse.Right.Down, function(self, h)
		cbh.ContextMenu("player") end, "Event.UI.Input.Mouse.Right.Down")

	cbh.playerframeloaded = true

	if not cbhPlayerValues.enabled then
		cbh.PlayerWindow:SetVisible(false)
	end
end



function cbh.PlayerFrameUpdate()
	if cbhPlayerValues.enabled then
		cbh.PlayerName:SetText(cbh.playerinfo.name)
		cbh.PlayerHealth:SetText(tostring(cbh.playerinfo.health))
		if cbh.playerinfo.role then cbh.PlayerRole:SetTexture("Rift", cbh.RoleImgs[cbh.playerinfo.role]) end
		cbh.PlayerCharges:SetText("P:"..cbh.playerinfo.planar.." V:"..cbh.playerinfo.vitality.."%")

		cbh.nameCalc(cbh.playerinfo.name, nil, "player")

		if cbh.playerinfo.role then
			cbh.PlayerRole:SetVisible(true)
			cbh.PlayerRole:SetTexture("Rift", cbh.RoleImgs[cbh.playerinfo.role])
		else cbh.PlayerRole:SetVisible(false) end

		if cbh.playerinfo.mark then
			cbh.PlayerRaidMark:SetVisible(true)
			cbh.PlayerRaidMark:SetTexture("Rift", cbh.RaidMarkerImages[cbh.playerinfo.mark])
		else cbh.PlayerRaidMark:SetVisible(false) end

		for i = 1, 4 do
			if cbh.playerinfo.calling == cbh.Calling[i] then
				cbh.PlayerName:SetFontColor(cbhCallingColors[i].r, cbhCallingColors[i].g, cbhCallingColors[i].b, 1)
				--cbh.PlayerHP:SetBackgroundColor(cbhCallingColors[i].r, cbhCallingColors[i].g, cbhCallingColors[i].b, 1)
			end
		end

		if cbh.playerinfo.mana then cbh.PlayerManaFrame:SetVisible(true) end
		if cbh.playerinfo.charge then cbh.PlayerChargeFrame:SetVisible(true) end

		local temptid = { [cbh.playerinfo.id] = cbh.playerinfo.health }
		cbh.PlayerUpdateHP(x, temptid)
	end
end

function cbh.PlayerCombatEnter()
	if cbhPlayerValues.enabled then
		cbh.PlayerCombat:SetVisible(true)
	end
end
function cbh.PlayerCombatLeave()
	if cbhPlayerValues.enabled then
		cbh.PlayerCombat:SetVisible(false)
	end
end

function cbh.PlayerUpdateHP(x, units)
	if units[cbh.playerinfo.id] then
		cbh.playerinfo.health = units[cbh.playerinfo.id]
			local healthcalc = (cbh.playerinfo.health / cbh.playerinfo.healthMax)
			local healthtick = (-healthcalc * cbhPlayerValues.fwidth + cbhPlayerValues.fwidth)
			cbh.PlayerHP:SetWidth(healthtick)
			if healthcalc < .3 then
				cbh.PlayerFrame:SetBackgroundColor(1, 0, 0, cbhCallingColors[7].a)
			elseif healthcalc < .65 then
				cbh.PlayerFrame:SetBackgroundColor(1, 1, 0, cbhCallingColors[7].a)
			else cbh.PlayerFrame:SetBackgroundColor(cbhCallingColors[7].r, cbhCallingColors[7].g, cbhCallingColors[7].b, cbhCallingColors[7].a) end

			-- if cbhValues.hidehealth == false then
				local health = 0
				local healthpercent = math.ceil(healthcalc * 100)
				if cbh.playerinfo.health > 1000000 then
					health = cbh.round((cbh.playerinfo.health/1000000), 1)
					health = tostring(health).."m"
					cbh.PlayerHealth:SetText(health.."  "..healthpercent.."%")
				elseif cbh.playerinfo.health > 1000 then
					health = cbh.round((cbh.playerinfo.health/1000), 1)
					health = tostring(health).."k"
					cbh.PlayerHealth:SetText(health.."  "..healthpercent.."%")
				elseif cbh.playerinfo.health <= 0 or math.ceil(healthcalc) == 0 then
					cbh.PlayerHealth:SetText("DEAD")
					cbh.playerinfo.dead = true
				end
			-- end
		-- end
	-- end
	end
end


function cbh.PlayerUpdateHPMax(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tvalue = units[cbh.playerinfo.id]
		cbh.playerinfo.healthMax = tvalue
		local tempid = {[cbh.playerinfo.id] = cbh.playerinfo.health}
		cbh.PlayerUpdateHP(x, tempid)
	end
end


function cbh.PlayerUpdateMana(x, units)
	if units[cbh.playerinfo.id] then
		cbh.playerinfo.mana = units[cbh.playerinfo.id]
		local manacalc = (cbh.playerinfo.mana / cbh.playerinfo.manaMax)
		local manatick = (-manacalc * cbhPlayerValues.fwidth + cbhPlayerValues.fwidth)
		cbh.PlayerMana:SetWidth(manatick)
	end
end


function cbh.PlayerUpdateManaMax(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tvalue = units[cbh.playerinfo.id]
		cbh.playerinfo.manaMax = tvalue
		local tempid = {[cbh.playerinfo.id] = cbh.playerinfo.mana}
		cbh.PlayerUpdateMana(x, tempid)
	end
end


function cbh.PlayerUpdateCharge(x, units)
	if units[cbh.playerinfo.id] then
		cbh.playerinfo.charge = units[cbh.playerinfo.id]
		local chargecalc = (cbh.playerinfo.charge / 100)
		local chargetick = (-chargecalc * cbhPlayerValues.fwidth + cbhPlayerValues.fwidth)
		cbh.PlayerCharge:SetWidth(chargetick)
	end
end


function cbh.PlayerUpdateChargeMax(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tvalue = units[cbh.playerinfo.id]
		cbh.playerinfo.chargeMax = tvalue
		local tempid = {[cbh.playerinfo.id] = cbh.playerinfo.mana}
		cbh.PlayerUpdateCharge(x, tempid)
	end
end




function cbh.PlayerUpdateLevel(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tvalue = units[cbh.playerinfo.id]
		cbh.playerinfo.level = tvalue
		cbh.nameCalc(cbh.playerinfo.name, nil, "player")
	end
end


function cbh.PlayerUpdateMark(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tvalue = units[cbh.playerinfo.id]
		cbh.PlayerRaidMark:SetVisible(true)
		cbh.PlayerRaidMark:SetTexture("Rift", cbh.RaidMarkerImages[tvalue])
	end
end


function cbh.PlayerMentored(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tuid = cbh.unitDetail("player")
		cbh.playerinfo.health = tuid.health
		cbh.playerinfo.healthMax = tuid.healthMax
		cbh.playerinfo.manaMax = tuid.manaMax
	end
end


function cbh.PlayerUpdatePlanar(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tvalue = units[cbh.playerinfo.id]
		cbh.playerinfo.planar = tvalue
		cbh.PlayerCharges:SetText("P:"..cbh.playerinfo.planar.."  V:"..cbh.playerinfo.vitality.."%")
	end
end


function cbh.PlayerUpdateVitality(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tvalue = units[cbh.playerinfo.id]
		cbh.playerinfo.vitality = tvalue
		cbh.PlayerCharges:SetText("P:"..cbh.playerinfo.planar.."  V:"..cbh.playerinfo.vitality.."%")
	end
end


function cbh.UnitRoleCheck(hEvent, units)
	if units[cbh.playerinfo.id] then
		local tvalue = units[cbh.playerinfo.id]
		cbh.PlayerRole:SetTexture("Rift", cbh.RoleImgs[tvalue])
	end
end


function cbh.PlayerToggle(t)
	if not t then
		cbhPlayerValues.enabled = true
		if not cbh.playerframeloaded then cbh.CreatePlayerFrame() cbh.PlayerFrameUpdate() end
		cbh.PlayerWindow:SetVisible(true)
	else
		cbhPlayerValues.enabled = false
		cbh.PlayerWindow:SetVisible(false)
	end
end

function cbh.TargetToggle(t)
	if not t then
		cbhTargetValues.enabled = true
		if not cbh.targetframeloaded then cbh.CreateTargetFrame() end
		if cbh.unitLookup("player.target") then
			cbh.TargetWindow:SetVisible(true)
			cbh.UnitStatus["target"] = nil
			cbh.GetTargetInfo(cbh.unitLookup("player.target"))
		end
	else
		cbhTargetValues.enabled = false
		cbh.TargetWindow:SetVisible(false)
	end
end

function cbh.UnitSlashCommands(hEvent, p)
	-- p = parameters passed after cbhu
	if p == "ptoggle" then
		cbh.PlayerToggle(cbhPlayerValues.enabled)
	elseif p == "ttoggle" then
		cbh.TargetToggle(cbhTargetValues.enabled)
	elseif p == "rplayer" then
		cbhPlayerValues = nil
	elseif p == "rtarget" then
		cbhPlayerValues = nil
	else
		print("----------------------------------------")
		print("CBH Unit Frames Help Menu")
		print("----------------------------------------")
		print("More options coming soon. \nUNIT FRAMES ARE IN BETA.\n")
		print("/cbhu ptoggle -- Toggles Player frame on/off")
		print("/cbhu ttoggle -- Toggles Target frame on/off")
		print("/cbhu rplayer -- Reset Player frame settings (REQUIRES RELOADUI)")
		print("/cbhu rtarget -- Reset Target frame settings (REQUIRES RELOADUI)")
		-- print("/cbh show	-- Show window")
		-- print("/cbh hide	-- Hide window")
		-- print("/cbh toggle	-- Toggles displaying main window")
		-- print("/cbh reset	-- Reset all settings AND clickbinds to default!")
		-- print("/cbh rbufflist	-- Reset tracked buffs")
		-- print("/cbh rbuffcolors	-- Reset all buff coloring to defaults")
		-- print("/cbh rcbhvalues	-- Reset location, texture, size, etc settings to defaults")
		-- print(" ")
		-- print("/cbh config  -- Display config window")
		-- print("/cbh lock -- Allows you to move the frame. Type again to lock")
		-- print("/cbh tt -- Toggles tooltips on frame mouseover")
	end
end

Command.Event.Attach(Command.Slash.Register("cbhu"), cbh.UnitSlashCommands, "SlashCommands")

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, cbh.PlayerLoadSettings, "PlayerLoadSettings")
Command.Event.Attach(Event.Addon.Startup.End, cbh.CreatePlayerFrame, "CreatePlayerFrame")

Command.Event.Attach(Event.System.Secure.Enter, cbh.PlayerCombatEnter, "PlayerCombatEnter")
Command.Event.Attach(Event.System.Secure.Leave, cbh.PlayerCombatLeave, "PlayerCombatLeave")

Command.Event.Attach(Event.Unit.Detail.Health, cbh.PlayerUpdateHP, "PlayerUpdateHP")
Command.Event.Attach(Event.Unit.Detail.HealthMax, cbh.PlayerUpdateHPMax, "PlayerUpdateHPMax")
Command.Event.Attach(Event.Unit.Detail.Mana, cbh.PlayerUpdateMana, "UpdateMana")
Command.Event.Attach(Event.Unit.Detail.ManaMax, cbh.PlayerUpdateManaMax, "UpdateManaMax")

Command.Event.Attach(Event.Unit.Detail.Charge, cbh.PlayerUpdateCharge, "UpdateCharge")
Command.Event.Attach(Event.Unit.Detail.ChargeMax, cbh.PlayerUpdateChargeMax, "UpdateChargeMax")
Command.Event.Attach(Event.Unit.Detail.Level, cbh.PlayerUpdateLevel, "PlayerUpdateLevel")
Command.Event.Attach(Event.Unit.Detail.Mark, cbh.PlayerUpdateMark, "PlayerUpdateMark")
Command.Event.Attach(Event.Unit.Detail.Planar, cbh.PlayerUpdatePlanar, "PlayerUpdatePlanar")
Command.Event.Attach(Event.Unit.Detail.Vitality, cbh.PlayerUpdateVitality, "PlayerUpdateVitality")
Command.Event.Attach(Event.Unit.Detail.Mentoring, cbh.PlayerMentored, "PlayerMentored")
Command.Event.Attach(Event.Unit.Detail.Role, cbh.UnitRoleCheck, "UpdateRole")
